@@ -127,6 +127,16 @@ type curlHTTPPost =
   | CURLFORM_CONTENTTYPE of string
 *)
 
+type curlSSLVersion =
+  | SSLVERSION_DEFAULT
+  | SSLVERSION_TLSv1
+  | SSLVERSION_SSLv2
+  | SSLVERSION_SSLv3
+  | SSLVERSION_TLSv1_0
+  | SSLVERSION_TLSv1_1
+  | SSLVERSION_TLSv1_2
+  | SSLVERSION_TLSv1_3
+
 type curlTimeCondition =
   | TIMECOND_NONE
   | TIMECOND_IFMODSINCE
@@ -153,6 +163,8 @@ type curlHTTPVersion =
   | HTTP_VERSION_NONE
   | HTTP_VERSION_1_0
   | HTTP_VERSION_1_1
+  | HTTP_VERSION_2
+  | HTTP_VERSION_2TLS
 
 type curlDebugType =
   | DEBUGTYPE_TEXT
@@ -160,6 +172,8 @@ type curlDebugType =
   | DEBUGTYPE_HEADER_OUT
   | DEBUGTYPE_DATA_IN
   | DEBUGTYPE_DATA_OUT
+  | DEBUGTYPE_SSL_DATA_IN
+  | DEBUGTYPE_SSL_DATA_OUT
   | DEBUGTYPE_END
 
 type curlAuth =
@@ -261,6 +275,13 @@ type curlProto =
 | CURLPROTO_RTMPTS
 | CURLPROTO_GOPHER
 
+(* sync curlPostRedir_table *)
+type curlPostRedir =
+| REDIR_POST_ALL
+| REDIR_POST_301
+| REDIR_POST_302
+| REDIR_POST_303
+
 type curlOption =
   | CURLOPT_WRITEFUNCTION of (string -> int)
   | CURLOPT_READFUNCTION of (int -> string)
@@ -313,7 +334,7 @@ type curlOption =
   | CURLOPT_POSTQUOTE of string list
   | CURLOPT_HEADERFUNCTION of (string -> int)
   | CURLOPT_COOKIEFILE of string
-  | CURLOPT_SSLVERSION of int
+  | CURLOPT_SSLVERSION of curlSSLVersion
   | CURLOPT_TIMECONDITION of curlTimeCondition
   | CURLOPT_TIMEVALUE of int32
   | CURLOPT_CUSTOMREQUEST of string
@@ -396,6 +417,13 @@ type curlOption =
   | CURLOPT_DNS_SERVERS of string
   | CURLOPT_MAIL_FROM of string
   | CURLOPT_MAIL_RCPT of string list
+  | CURLOPT_PIPEWAIT of bool
+  | CURLOPT_CERTINFO of bool
+  | CURLOPT_USERNAME of string
+  | CURLOPT_PASSWORD of string
+  | CURLOPT_LOGIN_OPTIONS of string
+  | CURLOPT_CONNECT_TO of string list
+  | CURLOPT_POSTREDIR of curlPostRedir list
 
 type initOption =
   | CURLINIT_GLOBALALL
@@ -440,69 +468,34 @@ type curlInfo =
   | CURLINFO_LOCAL_IP
   | CURLINFO_LOCAL_PORT
   | CURLINFO_CONDITION_UNMET
+  | CURLINFO_CERTINFO
 
 type curlInfoResult =
   | CURLINFO_String of string
   | CURLINFO_Long of int
   | CURLINFO_Double of float
   | CURLINFO_StringList of string list
+  | CURLINFO_StringListList of string list list
 
 exception NotImplemented of string
 
 let () = Callback.register_exception "Curl.NotImplemented" (NotImplemented "")
 
-external helper_global_init : initOption -> unit = "helper_curl_global_init"
-external helper_global_cleanup : unit -> unit = "helper_curl_global_cleanup"
-external helper_init : unit -> t = "helper_curl_easy_init"
-external helper_duphandle : t -> t = "helper_curl_easy_duphandle"
-external helper_setopt : t -> curlOption -> unit = "helper_curl_easy_setopt"
-external helper_perform : t -> unit = "helper_curl_easy_perform"
-external helper_cleanup : t -> unit = "helper_curl_easy_cleanup"
-external helper_getinfo : t -> curlInfo -> curlInfoResult = "helper_curl_easy_getinfo"
-external helper_escape : string -> string = "helper_curl_escape"
-external helper_unescape : string -> string = "helper_curl_unescape"
-external helper_getdate : string -> float -> float = "helper_curl_getdate"
-external helper_version : unit -> string = "helper_curl_version"
+external global_init : initOption -> unit = "caml_curl_global_init"
+external global_cleanup : unit -> unit = "caml_curl_global_cleanup"
+external init : unit -> t = "caml_curl_easy_init"
+external reset : t -> unit = "caml_curl_easy_reset"
+external duphandle : t -> t = "caml_curl_easy_duphandle"
+external setopt : t -> curlOption -> unit = "caml_curl_easy_setopt"
+external perform : t -> unit = "caml_curl_easy_perform"
+external cleanup : t -> unit = "caml_curl_easy_cleanup"
+external getinfo : t -> curlInfo -> curlInfoResult = "caml_curl_easy_getinfo"
+external escape : string -> string = "caml_curl_escape"
+external unescape : string -> string = "caml_curl_unescape"
+external getdate : string -> float -> float = "caml_curl_getdate"
+external version : unit -> string = "caml_curl_version"
 
-let init () =
-  helper_init ()
-
-external reset : t -> unit = "helper_curl_easy_reset"
-
-let duphandle conn =
-  helper_duphandle conn
-
-let setopt conn option =
-  helper_setopt conn option
-
-let perform conn =
-  helper_perform conn
-
-let cleanup conn =
-  helper_cleanup conn
-
-let getinfo conn =
-  helper_getinfo conn
-
-let global_init initOption =
-  helper_global_init initOption
-
-let global_cleanup () =
-  helper_global_cleanup ()
-
-let escape str =
-  helper_escape str
-    
-let unescape str =
-  helper_unescape str
-    
-let getdate str now =
-  helper_getdate str now
-    
-let version () =
-  helper_version ()
-
-type version_info = { 
+type version_info = {
   version : string;
   number : int * int * int;
   host : string;
@@ -519,7 +512,7 @@ type version_info = {
 
 external version_info : unit -> version_info = "caml_curl_version_info"
 
-external strerror : curlCode -> string = "helper_curl_easy_strerror"
+external strerror : curlCode -> string = "caml_curl_easy_strerror"
 let errno : curlCode -> int = Obj.magic
 
 type pauseOption = PAUSE_SEND | PAUSE_RECV | PAUSE_ALL
@@ -664,6 +657,9 @@ let set_sslengine conn sslengine =
 let set_sslenginedefault conn flag =
   setopt conn (CURLOPT_SSLENGINEDEFAULT flag)
 
+let set_certinfo conn flag =
+  setopt conn (CURLOPT_CERTINFO flag)
+
 let set_crlf conn flag =
   setopt conn (CURLOPT_CRLF flag)
 
@@ -930,6 +926,24 @@ let set_mailfrom conn l =
 let set_mailrcpt conn l =
   setopt conn (CURLOPT_MAIL_RCPT l)
 
+let set_pipewait conn b =
+  setopt conn (CURLOPT_PIPEWAIT b)
+
+let set_username conn s =
+  setopt conn (CURLOPT_USERNAME s)
+
+let set_password conn s =
+  setopt conn (CURLOPT_PASSWORD s)
+
+let set_login_options conn s =
+  setopt conn (CURLOPT_LOGIN_OPTIONS s)
+
+let set_connect_to conn l =
+  setopt conn (CURLOPT_CONNECT_TO l)
+
+let set_postredir conn l =
+  setopt conn (CURLOPT_POSTREDIR l)
+
 let get_effectiveurl conn =
   match (getinfo conn CURLINFO_EFFECTIVE_URL) with
   | CURLINFO_String s -> s
@@ -1048,6 +1062,11 @@ let get_httpconnectcode conn =
   |  CURLINFO_Long l -> l
   | _ -> 0
 
+let get_certinfo conn =
+  match (getinfo conn CURLINFO_CERTINFO) with
+  | CURLINFO_StringListList l -> l
+  | _ -> []
+
 let generate_auth auth =
   let result = ref [] in
     if auth land 1 != 0 then result := [CURLAUTH_BASIC];
@@ -1172,6 +1191,7 @@ class handle =
     method set_sslkeypasswd sslkeypasswd = set_sslkeypasswd conn sslkeypasswd
     method set_sslengine sslengine = set_sslengine conn sslengine
     method set_sslenginedefault flag = set_sslenginedefault conn flag
+    method set_certinfo flag = set_certinfo conn flag
     method set_crlf flag = set_crlf conn flag
     method set_quote quote = set_quote conn quote
     method set_postquote postquote = set_postquote conn postquote
@@ -1295,12 +1315,21 @@ class handle =
     method get_localip = get_localip conn
     method get_localport = get_localport conn
     method get_conditionunmet = get_conditionunmet conn
+    method get_certinfo = get_certinfo conn
 end
 
 module Multi = struct
 
   type mt
 
+  type curlPipelining = PIPE_NOTHING | PIPE_HTTP1 | PIPE_MULTIPLEX
+
+  type curlMultiOption =
+    | CURLMOPT_PIPELINING of curlPipelining list
+    | CURLMOPT_MAXCONNECTS of int
+    | CURLMOPT_MAX_PIPELINE_LENGTH of int
+    | CURLMOPT_MAX_HOST_CONNECTIONS of int
+
   exception Error of string
 
   let () = Callback.register_exception "Curl.Multi.Error" (Error "")
@@ -1308,7 +1337,8 @@ module Multi = struct
   external create : unit -> mt = "caml_curl_multi_init"
   external add : mt -> t -> unit = "caml_curl_multi_add_handle"
   external perform : mt -> int = "caml_curl_multi_perform_all"
-  external wait : mt -> bool = "caml_curlm_wait_data"
+  external wait : int -> mt -> bool = "caml_curl_multi_wait"
+  let wait ?(timeout_ms=1000) mt = wait timeout_ms mt
   external remove : mt -> t -> unit = "caml_curl_multi_remove_handle"
   external remove_finished : mt -> (t * curlCode) option = "caml_curlm_remove_finished"
   external cleanup : mt -> unit = "caml_curl_multi_cleanup"
@@ -1329,5 +1359,5 @@ module Multi = struct
 
   external timeout : mt -> int = "caml_curl_multi_timeout"
 
+  external setopt : mt -> curlMultiOption -> unit = "caml_curl_multi_setopt"
 end
-
