@@ -122,7 +122,7 @@ type curlHTTPPost =
   | CURLFORM_FILE of string * string * curlContentType
   | CURLFORM_BUFFER of string * string * string * curlContentType
 
-(*      
+(*
 type curlHTTPPost =
   | CURLFORM_COPYNAME of string
   | CURLFORM_PTRNAME of string
@@ -135,6 +135,16 @@ type curlHTTPPost =
   | CURLFORM_CONTENTTYPE of string
 *)
 
+type curlSSLVersion =
+  | SSLVERSION_DEFAULT (** attempt to figure out the remote SSL protocol version *)
+  | SSLVERSION_TLSv1 (** TLSv1.x *)
+  | SSLVERSION_SSLv2 (** SSLv2 *)
+  | SSLVERSION_SSLv3 (** SSLv3 *)
+  | SSLVERSION_TLSv1_0 (** Added in libcurl 7.34.0, will request TLSv1.x in previous versions *)
+  | SSLVERSION_TLSv1_1 (** Added in libcurl 7.34.0, will request TLSv1.x in previous versions *)
+  | SSLVERSION_TLSv1_2 (** Added in libcurl 7.34.0, will request TLSv1.x in previous versions *)
+  | SSLVERSION_TLSv1_3 (** Added in libcurl 7.52.0, will request TLSv1.x in previous versions *)
+
 type curlTimeCondition =
   | TIMECOND_NONE (** since 0.6.1 *)
   | TIMECOND_IFMODSINCE
@@ -158,9 +168,11 @@ type curlSSLVerifyHost =
   | SSLVERIFYHOST_HOSTNAME (** certificate must indicate the matching hostname, or the connection fails *)
 
 type curlHTTPVersion =
-  | HTTP_VERSION_NONE
-  | HTTP_VERSION_1_0
-  | HTTP_VERSION_1_1
+  | HTTP_VERSION_NONE (** libcurl will make the best choice *)
+  | HTTP_VERSION_1_0 (** please use HTTP 1.0 *)
+  | HTTP_VERSION_1_1 (** please use HTTP 1.1 *)
+  | HTTP_VERSION_2 (** please use HTTP 2 *)
+  | HTTP_VERSION_2TLS (** use version 2 for HTTPS, version 1.1 for HTTP *)
 
 type curlDebugType =
   | DEBUGTYPE_TEXT
@@ -168,6 +180,8 @@ type curlDebugType =
   | DEBUGTYPE_HEADER_OUT
   | DEBUGTYPE_DATA_IN
   | DEBUGTYPE_DATA_OUT
+  | DEBUGTYPE_SSL_DATA_IN
+  | DEBUGTYPE_SSL_DATA_OUT
   | DEBUGTYPE_END
 
 type curlAuth =
@@ -269,6 +283,13 @@ type curlProto =
 | CURLPROTO_RTMPTS
 | CURLPROTO_GOPHER
 
+(** if flag is not supported by libcurl - enabling it does nothing *)
+type curlPostRedir =
+| REDIR_POST_ALL
+| REDIR_POST_301
+| REDIR_POST_302
+| REDIR_POST_303 (** added in libcurl 7.26.0 *)
+
 type curlOption =
   | CURLOPT_WRITEFUNCTION of (string -> int)
   | CURLOPT_READFUNCTION of (int -> string)
@@ -321,7 +342,7 @@ type curlOption =
   | CURLOPT_POSTQUOTE of string list
   | CURLOPT_HEADERFUNCTION of (string -> int)
   | CURLOPT_COOKIEFILE of string
-  | CURLOPT_SSLVERSION of int
+  | CURLOPT_SSLVERSION of curlSSLVersion
   | CURLOPT_TIMECONDITION of curlTimeCondition
   | CURLOPT_TIMEVALUE of int32
   | CURLOPT_CUSTOMREQUEST of string
@@ -349,7 +370,7 @@ type curlOption =
   | CURLOPT_DNSCACHETIMEOUT of int
   | CURLOPT_DNSUSEGLOBALCACHE of bool
   | CURLOPT_DEBUGFUNCTION of (t -> curlDebugType -> string -> unit)
-  | CURLOPT_PRIVATE of string
+  | CURLOPT_PRIVATE of string (** doesn't employ libcurl's CURLOPT_PRIVATE *)
   | CURLOPT_HTTP200ALIASES of string list
   | CURLOPT_UNRESTRICTEDAUTH of bool
   | CURLOPT_FTPUSEEPRT of bool
@@ -404,6 +425,13 @@ type curlOption =
   | CURLOPT_DNS_SERVERS of string
   | CURLOPT_MAIL_FROM of string
   | CURLOPT_MAIL_RCPT of string list
+  | CURLOPT_PIPEWAIT of bool
+  | CURLOPT_CERTINFO of bool
+  | CURLOPT_USERNAME of string
+  | CURLOPT_PASSWORD of string
+  | CURLOPT_LOGIN_OPTIONS of string
+  | CURLOPT_CONNECT_TO of string list
+  | CURLOPT_POSTREDIR of curlPostRedir list
 
 type initOption =
   | CURLINIT_GLOBALALL
@@ -433,7 +461,7 @@ type curlInfo =
   | CURLINFO_CONTENT_TYPE
   | CURLINFO_REDIRECT_TIME
   | CURLINFO_REDIRECT_COUNT
-  | CURLINFO_PRIVATE
+  | CURLINFO_PRIVATE (** returns value previously set with CURLOPT_PRIVATE, doesn't employ libcurl's CURLINFO_PRIVATE *)
   | CURLINFO_HTTP_CONNECTCODE
   | CURLINFO_HTTPAUTH_AVAIL
   | CURLINFO_PROXYAUTH_AVAIL
@@ -448,14 +476,16 @@ type curlInfo =
   | CURLINFO_LOCAL_IP
   | CURLINFO_LOCAL_PORT
   | CURLINFO_CONDITION_UNMET
+  | CURLINFO_CERTINFO
 
 type curlInfoResult =
   | CURLINFO_String of string
   | CURLINFO_Long of int
   | CURLINFO_Double of float
   | CURLINFO_StringList of string list
+  | CURLINFO_StringListList of string list list
 
-type version_info = { 
+type version_info = {
   version : string;
   number : int * int * int;
   host : string;
@@ -552,12 +582,13 @@ val set_sslkeytype : t -> string -> unit
 val set_sslkeypasswd : t -> string -> unit
 val set_sslengine : t -> string -> unit
 val set_sslenginedefault : t -> bool -> unit
+val set_certinfo : t -> bool -> unit
 val set_crlf : t -> bool -> unit
 val set_quote : t -> string list -> unit
 val set_postquote : t -> string list -> unit
 val set_headerfunction: t -> (string -> int) -> unit
 val set_cookiefile : t -> string -> unit
-val set_sslversion : t -> int -> unit
+val set_sslversion : t -> curlSSLVersion -> unit
 val set_timecondition : t -> curlTimeCondition -> unit
 val set_timevalue : t -> int32 -> unit
 val set_customrequest : t -> string -> unit
@@ -646,6 +677,17 @@ val set_resolve : t -> (string * int * string) list -> (string * int) list -> un
 val set_dns_servers : t -> string list -> unit
 val set_mailfrom : t -> string -> unit
 val set_mailrcpt : t -> string list -> unit
+val set_pipewait : t -> bool -> unit
+val set_username : t -> string -> unit
+(** @since 0.8.0 *)
+val set_password : t -> string -> unit
+(** @since 0.8.0 *)
+val set_login_options : t -> string -> unit
+(** @since 0.8.0 *)
+val set_connect_to : t -> string list -> unit
+(** @since 0.8.0 *)
+val set_postredir : t -> curlPostRedir list -> unit
+(** @since 0.8.1 *)
 
 (** {2 Get transfer properties} *)
 
@@ -688,6 +730,7 @@ val get_localport : t -> int
 (** @since 0.5.5 (libcurl 7.21.0) *)
 val get_conditionunmet : t -> bool
 (** @since 0.6.1 (libcurl 7.19.4) *)
+val get_certinfo : t -> string list list
 
 (** {2 Object interface} *)
 
@@ -745,12 +788,13 @@ class handle :
     method set_sslkeypasswd : string -> unit
     method set_sslengine : string -> unit
     method set_sslenginedefault : bool -> unit
+    method set_certinfo : bool -> unit
     method set_crlf : bool -> unit
     method set_quote : string list -> unit
     method set_postquote : string list -> unit
     method set_headerfunction: (string -> int) -> unit
     method set_cookiefile : string -> unit
-    method set_sslversion : int -> unit
+    method set_sslversion : curlSSLVersion -> unit
     method set_timecondition : curlTimeCondition -> unit
     method set_timevalue : int32 -> unit
     method set_customrequest : string -> unit
@@ -798,7 +842,7 @@ class handle :
     method set_tcpnodelay : bool -> unit
     method set_ftpsslauth : curlFTPSSLAuth -> unit
     method set_ioctlfunction : (t -> curlIOCmd -> curlIOErr) -> unit
-    method set_ftpaccount : string -> unit 
+    method set_ftpaccount : string -> unit
     method set_cookielist : string -> unit
     method set_ignorecontentlength : bool -> unit
     method set_ftpskippasvip : bool -> unit
@@ -867,6 +911,7 @@ class handle :
     method get_localip : string
     method get_localport : int
     method get_conditionunmet : bool
+    method get_certinfo : string list list
   end
 
 (** {2 curl_multi API} *)
@@ -877,6 +922,14 @@ module Multi : sig
   (** type of Curl multi stack *)
   type mt
 
+  type curlPipelining = PIPE_NOTHING | PIPE_HTTP1 | PIPE_MULTIPLEX
+
+  type curlMultiOption =
+    | CURLMOPT_PIPELINING of curlPipelining list
+    | CURLMOPT_MAXCONNECTS of int
+    | CURLMOPT_MAX_PIPELINE_LENGTH of int
+    | CURLMOPT_MAX_HOST_CONNECTIONS of int
+
   (** exception raised on internal errors *)
   exception Error of string
 
@@ -895,7 +948,7 @@ module Multi : sig
 
   (** wait till there are some active data transfers on multi stack
       @return whether [perform] should be called *)
-  val wait : mt -> bool
+  val wait : ?timeout_ms:int -> mt -> bool
 
   (** remove finished handle from the multi stack if any. The returned handle may be reused *)
   val remove_finished : mt -> (t * curlCode) option
@@ -904,7 +957,7 @@ module Multi : sig
   val cleanup : mt -> unit
 
   (** events that should be reported for the socket *)
-  type poll = 
+  type poll =
     | POLL_NONE    (** none *)
     | POLL_IN      (** available for reading *)
     | POLL_OUT     (** available for writing *)
@@ -912,7 +965,7 @@ module Multi : sig
     | POLL_REMOVE  (** socket not needed anymore *)
 
   (** socket status *)
-  type fd_status = 
+  type fd_status =
     | EV_AUTO  (** determine socket status automatically (with extra system call) *)
     | EV_IN    (** socket has incoming data *)
     | EV_OUT   (** socket is available for writing *)
@@ -930,12 +983,12 @@ module Multi : sig
 
   (** perform pending data transfers (if any) on all handles currently in multi stack
       (not recommended, {!action} should be used instead)
-      @return the number of handles that still transfer data 
+      @return the number of handles that still transfer data
       @raise Error on errors
   *)
   val action_all : mt -> int
 
-  (** inform libcurl that timeout occured 
+  (** inform libcurl that timeout occured
       @raise Error on errors
   *)
   val action_timeout : mt -> unit
@@ -953,4 +1006,7 @@ module Multi : sig
   *)
   external timeout : mt -> int = "caml_curl_multi_timeout"
 
+  (** @raise NotImplemented for not implemented option *)
+  val setopt : mt -> curlMultiOption -> unit
+
 end
